<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Services\MapPortaLoja as MapPortaLojaService;
use \Exception as Exception;


class MapPortaLoja extends Crud {
    protected $service;
    protected $dataSourceManager;
    protected $dataSourceName = 'mapportaloja';
    protected $dataSourceNameImpressora = 'impressoraNFLoja';
    
    public function __construct (Manager $dataSourceManager, MapPortaLojaService $service) {
        $this->dataSourceManager = $dataSourceManager;
        $this->service = $service;
    }
    
    public function findMap(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'select_mapportaloja';
        parent::find($request,$response);
    }
    
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            
            $row = $dataSet->getRows()[0];
            
            $row['IDATIVO'] = 'S';
            $filterCriteria = new FilterCriteria($this->dataSourceName);
            $filterCriteria->addCondition('NRORG', FilterCriteria::EQ, $row['NRORG'] );
            $filterCriteria->addCondition('CDPORTAIMPR', FilterCriteria::EQ, $row['CDPORTAIMPR'] );
            $filterCriteria->addCondition('CDFILIAL', FilterCriteria::EQ, $row['CDFILIAL'] );
            $filterCriteria->addCondition('CDLOJA', FilterCriteria::EQ, $row['CDLOJA'] );
            
            $dataSet2 = $this->dataSourceManager->findBy($filterCriteria);
            
            
            $row['__is_new'] = (count($dataSet2->getRows()) !== 1);
            
            if( $row['__is_new'] ) {
                $row['DTULTATU'] = null;
                $row['NRORGULTATU'] = null; 
                $row['CDOPERULTATU'] = null;
            }
            
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getDistinctPortas(DTO\Request\Filter $request, DTO\Response $response) {
        try {  
            $result = $this->service->find($request->getFilterCriteria(), $this->dataSourceManager);
            $response->addDataSet(new DataSet($this->dataSourceName, $result));
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
}
//MAPEAMENTO DE PORTAS POR LOJA (E POR IMPRESSORA)